@@ -17,6 +17,52 @@ import damo_version
 
 # Core data structures
 
+class OpsAttrs:
+    use_reports = None
+    write_only = None
+    cpus = None         #cpus list string
+    tids = None         #thread ids list string
+
+    def __init__(self, use_reports=False, write_only=False, cpus=None,
+                 tids=None):
+        self.use_reports = _damo_fmt_str.text_to_bool(use_reports)
+        self.write_only = _damo_fmt_str.text_to_bool(write_only)
+        self.cpus = cpus if cpus is not None else 'all'
+        self.tids = tids if tids is not None else ''
+
+    def to_str(self, raw):
+        return 'use_reports: %s, write_only: %s, cpus: %s, threads: %s' % (
+                self.use_reports, self.write_only, self.cpus, self.tids)
+
+    def __str__(self):
+        return self.to_str(False)
+
+    def __eq__(self, other):
+        return type(self) == type(other) and \
+                self.use_reports == other.use_reports and \
+                self.write_only == other.write_only and \
+                self.cpus == other.cpus and \
+                self.tids == other.tids
+
+    @classmethod
+    def from_kvpairs(cls, kvpairs):
+        use_reports = kvpairs['use_reports'] \
+                if 'use_reports' in kvpiars else False
+        write_only = kvpairs['write_only'] \
+                if 'write_only' in kvpairs else False
+        cpus = kvpairs['cpus'] if 'cpus' in kvpairs else 'all'
+        tids = kvpairs['tids'] if 'tids' in kvpairs else ''
+        return OpsAttrs(use_reports=use_reports, write_only=write_only,
+                        cpus=cpus, tids=tids)
+
+    def to_kvpairs(self, raw=False):
+        return collections.OrderedDict([
+            ('use_reports', self.use_reports),
+            ('write_only', self.write_only),
+            ('cpus', self.cpus),
+            ('tids', self.tids),
+            ])
+
 class DamonIntervalsGoal:
     access_bp = None
     aggrs = None
@@ -147,6 +193,7 @@ unit_aggr_intervals = 'aggr_intervals'
 class DamonNrAccesses:
     samples = None
     percent = None
+    hz = None
 
     def __init__(self, val, unit):
         if val == None or unit == None:
@@ -161,7 +208,15 @@ class DamonNrAccesses:
     def __eq__(self, other):
         return (type(self) == type(other) and
                 ((self.samples != None and self.samples == other.samples) or
-                    (self.percent != None and self.percent == other.percent)))
+                    (self.percent != None and self.percent == other.percent)
+                 ))
+
+    def in_hz(self, aggr_us):
+        return self.samples / (aggr_us / 1000000)
+
+    def in_percent(self, sample_us, aggr_us):
+        max_samples = aggr_us / sample_us
+        return self.samples * 100.0 / max_samples
 
     def add_unset_unit(self, intervals):
         if self.samples != None and self.percent != None:
@@ -169,7 +224,7 @@ class DamonNrAccesses:
         max_val = intervals.aggr / intervals.sample
         if self.samples == None:
             self.samples = int(self.percent * max_val / 100)
-        elif self.percent == None:
+        if self.percent == None:
             self.percent = int(self.samples * 100.0 / max_val)
 
     def to_str(self, unit, raw):
@@ -191,7 +246,8 @@ class DamonNrAccesses:
 
     def to_kvpairs(self, raw=False):
         return collections.OrderedDict(
-                [('samples', self.samples), ('percent', self.percent)])
+                [('samples', self.samples), ('percent', self.percent),
+                 ])
 
 class DamonAge:
     usec = None
@@ -483,6 +539,36 @@ class DamosAccessPattern:
                 other.converted_for_units(
                     unit_samples, unit_aggr_intervals, intervals))
 
+class DamosDest:
+    id = None
+    weight = None
+
+    def __init__(self, id=0, weight=0):
+        self.id = _damo_fmt_str.text_to_nr(id)
+        self.weight = _damo_fmt_str.text_to_nr(weight)
+
+    def to_str(self, raw):
+        return 'id: %s, weight: %s' % (
+                _damo_fmt_str.format_nr(self.id, raw),
+                _damo_fmt_str.format_nr(self.weight, raw))
+
+    def __str__(self):
+        return self.to_str(False)
+
+    def __eq__(self, other):
+        return (type(self) == type(other) and
+                self.id == other.id and self.weight == other.weight)
+
+    @classmethod
+    def from_kvpairs(cls, kv):
+        return DamosDest(kv['id'], kv['weight'])
+
+    def to_kvpairs(self, raw=False):
+        return collections.OrderedDict([
+            ('id', self.id),
+            ('weight', self.weight),
+            ])
+
 qgoal_user_input = 'user_input'
 qgoal_some_mem_psi_us = 'some_mem_psi_us'
 qgoal_node_mem_used_bp = 'node_mem_used_bp'
@@ -910,6 +996,7 @@ class Damos:
     access_pattern = None
     action = None
     target_nid = None
+    dests = None
     apply_interval_us = None
     quotas = None
     watermarks = None
@@ -925,12 +1012,13 @@ class Damos:
             self, access_pattern=None,
             action=damos_action_stat, target_nid=None, apply_interval_us=None,
             quotas=None, watermarks=None, filters=None, stats=None,
-            tried_regions=None, tried_bytes=None):
+            tried_regions=None, tried_bytes=None, dests=None):
         self.access_pattern = (access_pattern
                 if access_pattern != None else DamosAccessPattern())
         if not action in damos_actions:
             raise Exception('wrong damos action: %s' % action)
         self.action = action
+        self.dests = dests if dests is not None else []
         self.target_nid = target_nid
         if apply_interval_us != None:
             self.apply_interval_us = _damo_fmt_str.text_to_us(
@@ -971,6 +1059,8 @@ class Damos:
             lines.append('target access pattern')
             lines.append(_damo_fmt_str.indent_lines(
                 self.access_pattern.to_str(raw), 4))
+        for dest in self.dests:
+            lines.append('dest %d weight %d' % (dest.id, dest.weight))
         if self.quotas is not None:
             lines.append('quotas')
             lines.append(_damo_fmt_str.indent_lines(
@@ -1003,6 +1093,7 @@ class Damos:
         return (type(self) == type(other) and
                 self.access_pattern == other.access_pattern and
                 self.action == other.action and
+                self.dests == other.dests and
                 self.apply_interval_us == other.apply_interval_us and
                 self.quotas == other.quotas and
                 self.watermarks == other.watermarks and
@@ -1014,6 +1105,10 @@ class Damos:
         if 'filters' in kv:
             for damos_filter_kv in kv['filters']:
                 filters.append(DamosFilter.from_kvpairs(damos_filter_kv))
+        dests = []
+        if 'dests' in kv:
+            for dest_kv in kv['dests']:
+                dests.append(DamosDest.from_kvpairs(dest_kv))
         return Damos(DamosAccessPattern.from_kvpairs(kv['access_pattern'])
                     if 'access_pattern' in kv else DamosAccessPattern(),
                 kv['action'] if 'action' in kv else damos_action_stat,
@@ -1024,16 +1119,17 @@ class Damos:
                 DamosWatermarks.from_kvpairs(kv['watermarks'])
                     if 'watermarks' in kv else DamosWatermarks(),
                 filters,
-                None, None)
+                None, None, dests=dests)
 
     def to_kvpairs(self, raw=False, omit_defaults=False, params_only=False):
         kv = collections.OrderedDict()
         kv['action'] = self.action
+        kv['dests'] = [dest.to_kvpairs(raw) for dest in self.dests]
         if is_damos_migrate_action(self.action):
             kv['target_nid'] = self.target_nid
         if not omit_defaults or self.access_pattern != DamosAccessPattern():
             kv['access_pattern'] = self.access_pattern.to_kvpairs(raw)
-        kv['apply_interval_us'] = self.apply_interval_us
+        kv['apply_interval_us'] = _damo_fmt_str.format_time_us(self.apply_interval_us, raw)
         if not omit_defaults or self.quotas != DamosQuotas():
             kv['quotas'] = self.quotas.to_kvpairs(raw)
         if not omit_defaults or self.watermarks != DamosWatermarks():
@@ -1052,6 +1148,7 @@ class Damos:
                 self.access_pattern.effectively_equal(
                     other.access_pattern, intervals) and
                 self.action == other.action and
+                self.dests == other.dests and
                 self.apply_interval_us == other.apply_interval_us and
                 self.quotas == other.quotas and
                 self.watermarks == other.watermarks and
@@ -1059,7 +1156,7 @@ class Damos:
 
 class DamonCtx:
     ops = None
-    addr_unit = None
+    ops_attrs = None
     targets = None
     intervals = None
     nr_regions = None
@@ -1067,11 +1164,9 @@ class DamonCtx:
     kdamond = None
 
     def __init__(self, ops='paddr', targets=None, intervals=None,
-                 nr_regions=None, schemes=None, addr_unit=None):
+                 nr_regions=None, schemes=None, ops_attrs=None):
         self.ops = ops
-        if addr_unit is not None:
-            self.addr_unit = _damo_fmt_str.text_to_bytes(addr_unit)
-        self.addr_unit = addr_unit
+        self.ops_attrs = ops_attrs if ops_attrs is not None else OpsAttrs()
         self.targets = targets if targets is not None else []
         for target in self.targets:
             target.context = self
@@ -1084,11 +1179,14 @@ class DamonCtx:
             scheme.context = self
 
     def to_str(self, raw, params_only=False):
-        ops_line = 'ops: %s' % self.ops
-        if self.addr_unit is not None:
-            ops_line = '%s (addr_unit %s)' % (
-                    ops_line, _damo_fmt_str.format_sz(self.addr_unit, raw))
-        lines = [ops_line]
+        ops_line_tokens = ['ops: %s' % self.ops]
+        if self.ops_attrs.use_reports:
+            ops_line_tokens.append('use_reports')
+            ops_line_tokens.append('cpus %s' % self.ops_attrs.cpus)
+            ops_line_tokens.append('threads %s' % self.ops_attrs.tids)
+        if self.ops_attrs.write_only:
+            ops_line_tokens.append('write-only')
+        lines = [' '.join(ops_line_tokens)]
         for idx, target in enumerate(self.targets):
             lines.append('target %d' % idx)
             lines.append(_damo_fmt_str.indent_lines(target.to_str(raw), 4))
@@ -1125,15 +1223,12 @@ class DamonCtx:
                 DamonNrRegionsRange.from_kvpairs(kv['nr_regions'])
                     if 'nr_regions' in kv else DAmonNrRegionsRange(),
                 [Damos.from_kvpairs(s) for s in kv['schemes']]
-                    if 'schemes' in kv else [],
-                    kv['addr_unit']
-                    if 'addr_unit' in kv else None)
+                    if 'schemes' in kv else [])
         return ctx
 
     def to_kvpairs(self, raw=False, omit_defaults=False, params_only=False):
         kv = collections.OrderedDict({})
         kv['ops'] = self.ops
-        kv['addr_unit'] = self.addr_unit
         kv['targets'] = [t.to_kvpairs(raw) for t in self.targets]
         if not omit_defaults or self.intervals != DamonIntervals():
             kv['intervals'] = self.intervals.to_kvpairs(raw)
@@ -1149,29 +1244,36 @@ def target_has_pid(ops):
 class Kdamond:
     state = None
     pid = None
+    refresh_ms = None
     contexts = None
 
-    def __init__(self, state, pid, contexts):
+    def __init__(self, state, pid, contexts, refresh_ms=0):
         self.state = state
         self.pid = pid
+        self.refresh_ms = _damo_fmt_str.text_to_ms(refresh_ms)
         self.contexts = contexts
         for ctx in self.contexts:
             ctx.kdamond = self
 
     def summary_str(self, show_cpu=False, params_only=False,
-                    omit_defaults=False):
+                    omit_defaults=False, raw_number=False):
         words = []
         if params_only is False and self.state is not None:
             words.append('state: %s' % self.state)
         if params_only is False and self.pid is not None:
             words.append('pid: %s' % self.pid)
+        if self.refresh_ms != 0:
+            words.append('stats refresh per: %s' %
+                         _damo_fmt_str.format_time_ms(
+                             self.refresh_ms, raw_number))
         if show_cpu:
             words.append('cpu usage: %s' % self.get_cpu_usage())
         return ', '.join(words)
 
     def to_str(self, raw, show_cpu=False, params_only=False):
         lines = []
-        summary_line = self.summary_str(show_cpu, params_only)
+        summary_line = self.summary_str(show_cpu, params_only,
+                                        omit_defaults=False, raw_number=raw)
         if summary_line != '':
             lines.append(summary_line)
         for idx, ctx in enumerate(self.contexts):
@@ -1203,13 +1305,16 @@ class Kdamond:
         return Kdamond(
                 kv['state'] if 'state' in kv else 'off',
                 kv['pid'] if 'pid' in kv else None,
-                [DamonCtx.from_kvpairs(c) for c in kv['contexts']])
+                [DamonCtx.from_kvpairs(c) for c in kv['contexts']],
+                refresh_ms=kv['refresh_ms'] if 'refresh_ms' in kv else 0,
+                )
 
     def to_kvpairs(self, raw=False, omit_defaults=False, params_only=False):
         kv = collections.OrderedDict()
         if not params_only:
             kv['state'] = self.state
             kv['pid'] = self.pid
+        kv['refresh_ms'] = self.refresh_ms
         kv['contexts'] = [c.to_kvpairs(raw, omit_defaults, params_only)
                           for c in self.contexts]
         return kv
@@ -1281,10 +1386,11 @@ features = ['record',       # was in DAMON patchset, but not merged in mainline
                                         # merged in v6.15-rc1
             'schemes_filters_active',
                                         # merged in v6.15-rc1
-
             'schemes_quota_goal_node_mem_used_free',
-                                        # under development
-            'addr_unit'                 # under development
+                                        # merged in v6.16-rc1
+            'schemes_dests',            # merged in v6.17-rc1
+            'sysfs_refresh_ms',         # merged in v6.17-rc1
+            'ops_attrs',                # hacking on damon/next
             ]
 
 _damon_fs = None
@@ -1464,6 +1570,11 @@ def damon_interface():
 def stage_kdamonds(kdamonds):
     return _damon_fs.stage_kdamonds(kdamonds)
 
+def stage_kdamonds_targets(kdamonds):
+    if _damon_fs == _damon_dbgfs:
+        return 'debugfs interface does not support stage_kdamonds_target'
+    return _damon_fs.stage_kdamonds_targets(kdamonds)
+
 def commit_staged(kdamond_idxs):
     if _damon_fs == _damon_dbgfs:
         return 'debugfs interface does not support commit_staged()'
@@ -1474,11 +1585,15 @@ def commit_quota_goals(kdamond_idxs):
         return 'debugfs interface does not support commit_quota_goals()'
     return _damon_fs.commit_quota_goals(kdamond_idxs)
 
-def commit(kdamonds, commit_quota_goals_only=False):
-    if not commit_quota_goals_only:
+def commit(kdamonds, commit_quota_goals_only=False, commit_targets_only=False):
+    if not commit_quota_goals_only and not commit_targets_only:
         err = stage_kdamonds(kdamonds)
         if err:
             return 'staging updates failed (%s)' % err
+    elif commit_targets_only:
+        err = stage_kdamonds_targets(kdamonds)
+        if err:
+            return 'staging target updates failed (%s)' % err
 
     kdamond_idxs = ['%s' % idx for idx, k in enumerate(kdamonds)]
 
