@@ -303,7 +303,7 @@ def damos_options_to_quotas(quotas, goals):
     return quotas, None
 
 def damos_options_to_scheme(sz_region, access_rate, age, action,
-        apply_interval, quotas, goals, wmarks, target_nid, filters):
+        apply_interval, quotas, goals, wmarks, target_nid, filters, dests):
     if quotas != None:
         quotas, err = damos_options_to_quotas(quotas, goals)
         if err is not None:
@@ -326,7 +326,7 @@ def damos_options_to_scheme(sz_region, access_rate, age, action,
                 access_pattern=_damon.DamosAccessPattern(sz_region,
                     access_rate, _damon.unit_percent, age, _damon.unit_usec),
                 action=action, target_nid=target_nid,
-                apply_interval_us=apply_interval, quotas=quotas,
+                dests=dests, apply_interval_us=apply_interval, quotas=quotas,
                 watermarks=wmarks, filters=filters), None
     except Exception as e:
         return None, 'Wrong \'--damos_*\' argument (%s)' % e
@@ -383,16 +383,29 @@ def damos_options_to_schemes(args):
     args.damos_quotas += [None] * (nr_schemes - len(args.damos_quotas))
     args.damos_wmarks += [None] * (nr_schemes - len(args.damos_wmarks))
     target_nid = [None] * nr_schemes
+    dests_list = []
+    for i in range(nr_schemes):
+        dests_list.append([])
     schemes = []
 
     for i in range(nr_schemes):
         action = args.damos_action[i][0]
         if _damon.is_damos_migrate_action(action):
             try:
-                target_nid[i] = int(args.damos_action[i][1])
-                args.damos_action[i] = args.damos_action[i][0]
+                if len(args.damos_action[i]) == 2:
+                    target_nid[i] = int(args.damos_action[i][1])
+                    args.damos_action[i] = args.damos_action[i][0]
+                elif len(args.damos_action[i]) > 2:
+                    dests = []
+                    for j in range(1, len(args.damos_action[i]), 2):
+                        dests.append(_damon.DamosDest(
+                            args.damos_action[i][j],
+                            args.damos_action[i][j + 1]))
+                    dests_list[i] = dests
+                    args.damos_action[i] = args.damos_action[i][0]
             except:
-                return [], '"%s" action require a numeric target_nid argument.' \
+                return [], '"%s" action require a numeric target_nid ' \
+                        'or dests arguments.' \
                             % args.damos_action[i][0]
         else:
             args.damos_action[i] = action
@@ -408,10 +421,12 @@ def damos_options_to_schemes(args):
             filter_start_index = sum(args.damos_nr_filters[:i])
             filter_end_index = filter_start_index + args.damos_nr_filters[i]
             filters = args.damos_filter[filter_start_index:filter_end_index]
-        scheme, err = damos_options_to_scheme(args.damos_sz_region[i],
+        scheme, err = damos_options_to_scheme(
+                args.damos_sz_region[i],
                 args.damos_access_rate[i], args.damos_age[i],
                 args.damos_action[i], args.damos_apply_interval[i],
-                args.damos_quotas[i], qgoals, args.damos_wmarks[i], target_nid[i], filters)
+                args.damos_quotas[i], qgoals, args.damos_wmarks[i],
+                target_nid[i], filters, dests_list[i])
         if err != None:
             return [], err
         schemes.append(scheme)
@@ -466,10 +481,32 @@ def damon_ctx_for(args, idx):
     except Exception as e:
         return None, 'invalid nr_regions arguments (%s)' % e
     ops = args.ops[idx]
+    if args.exp_ops_use_reports[idx] is not None:
+        use_reports = args.exp_ops_use_reports[idx]
+    else:
+        use_reports = False
+    if args.exp_ops_write_only[idx] is not None:
+        write_only = args.exp_ops_write_only[idx]
+    else:
+        write_only = False
+    if args.exp_ops_cpus[idx] is not None:
+        cpus = args.exp_ops_cpus[idx]
+    else:
+        cpus = 'all'
+    if args.exp_ops_tids[idx] is not None:
+        tids = args.exp_ops_tids[idx]
+    else:
+        tids = ''
+    try:
+        ops_attrs = _damon.OpsAttrs(
+                use_reports=use_reports, write_only=write_only, cpus=cpus,
+                tids=tids)
+    except Exception as e:
+        return None, 'invalid ops_attrs arguments (%s)' % e
 
     try:
         ctx = _damon.DamonCtx(ops, None, intervals, nr_regions, schemes=[],
-                              addr_unit=args.ops_addr_unit)
+                              ops_attrs=ops_attrs)
         return ctx, None
     except Exception as e:
         return None, 'Creating context from arguments failed (%s)' % e
@@ -501,7 +538,8 @@ def get_nr_targets(args):
 def fillup_none_ctx_args(args):
     nr_ctxs = get_nr_ctxs(args)
     for attr_name in [
-            'ops', 'sample', 'aggr', 'updr', 'minr', 'maxr',
+            'ops', 'exp_ops_use_reports', 'exp_ops_write_only', 'exp_ops_cpus',
+            'exp_ops_tids', 'sample', 'aggr', 'updr', 'minr', 'maxr',
             'monitoring_intervals', 'monitoring_intervals_goal',
             'monitoring_nr_regions_range']:
         attr_val = getattr(args, attr_name)
@@ -736,9 +774,15 @@ def kdamonds_for(args):
     kdamonds = []
     ctx_idx = 0
     for nr in args.nr_ctxs:
+        if (args.refresh_stat is not None and
+            len(args.refresh_stat) > len(kdamonds)):
+            refresh_ms = args.refresh_stat[len(kdamonds)]
+        else:
+            refresh_ms = 0
         try:
             kdamonds.append(_damon.Kdamond(
-                state=None, pid=None, contexts=ctxs[ctx_idx:ctx_idx + nr]))
+                state=None, pid=None, refresh_ms=refresh_ms,
+                contexts=ctxs[ctx_idx:ctx_idx + nr]))
         except Exception as e:
             return None, 'kdamond creation fail (%s)' % e
         ctx_idx += nr
@@ -828,14 +872,30 @@ def set_monitoring_attrs_argparser(parser, hide_help=False):
                         help='min/max number of monitoring regions'
                         if not hide_help else argparse.SUPPRESS)
 
-def set_monitoring_argparser(parser, hide_help=False):
+def set_monitoring_damos_common_args(parser, hide_help=False):
     parser.add_argument('--ops', choices=['vaddr', 'paddr', 'fvaddr'],
                         action='append',
                         help='monitoring operations set'
                         if not hide_help else argparse.SUPPRESS)
-    parser.add_argument('--ops_addr_unit', metavar='<bytes>',
-                        help='operations set address unit'
+    parser.add_argument('--exp_ops_use_reports', action='append',
+                        metavar='<Y|N>',
+                        help='use access reports (experimental)'
                         if not hide_help else argparse.SUPPRESS)
+    parser.add_argument('--exp_ops_write_only', action='append',
+                        metavar='<Y|N>',
+                        help='monitor writes only (experimental)'
+                        if not hide_help else argparse.SUPPRESS)
+    parser.add_argument('--exp_ops_cpus', action='append', metavar='<cpulist>',
+                        help='monitor for given cpus only (experimental)'
+                        if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--exp_ops_tids', action='append', metavar='<thread ids list>',
+            help='monitoring for given threads only (experimental)')
+    parser.add_argument(
+            '--refresh_stat', metavar='<milliseconds>', action='append',
+            help='automatic kdamond internal stat refresh interval')
+
+def set_monitoring_argparser(parser, hide_help=False):
     parser.add_argument('--target_pid', type=int, metavar='<pid>',
                         action='append',
                         help='pid of monitoring target process'
@@ -865,6 +925,10 @@ def set_damos_argparser(parser, hide_help):
                             'damos action to apply to the target regions.',
                             '<action> should be {%s}.' %
                             ','.join(_damon.damos_actions),
+                            'for \'migrate_{hot,cold}\' actions,',
+                            'single target node id or',
+                            'multiple destination nodes input',
+                            'should also be given.'
                             ])
                         if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--damos_sz_region', metavar=('<min>', '<max>'),
@@ -960,6 +1024,7 @@ def set_misc_damon_params_argparser(parser):
                 'DAMON parameters (same to that for --kdamonds)']))
 
 def set_damon_params_argparser(parser, min_help):
+    set_monitoring_damos_common_args(parser, min_help)
     set_monitoring_argparser(parser, min_help)
     set_damos_argparser(parser, min_help)
     set_misc_damon_params_argparser(parser)
