@@ -191,54 +191,67 @@ snapshot_formatters = [
                 lambda snapshot, record, fmt:
                 damos_stats_str(snapshot, record, fmt),
                 'DAMOS stats for the snapshot-retrieval scheme'),
+        Formatter(
+                '<snapshot intervals>',
+                lambda snapshot, record, fmt:
+                snapshot_intervals_str(snapshot, record, fmt),
+                'Snapshot intervals status'),
         ]
 
 region_formatters = [
         Formatter(
             '<index>',
-            lambda index, region, fmt:
+            lambda index, region, snapshot, record, fmt:
             _damo_fmt_str.format_nr(index, fmt.raw_number),
             'index of the region in the regions of the snapshot'),
         Formatter(
             '<start address>',
-            lambda index, region, fmt:
+            lambda index, region, snapshot, record, fmt:
             _damo_fmt_str.format_sz(region.start, fmt.raw_number),
             'start address of the region'),
         Formatter(
             '<end address>',
-            lambda index, region, fmt:
+            lambda index, region, snapshot, record, fmt:
             _damo_fmt_str.format_sz(region.end, fmt.raw_number),
             'end address of the region'),
         Formatter(
             '<size>',
-            lambda index, region, fmt:
+            lambda index, region, snapshot, record, fmt:
             _damo_fmt_str.format_sz(region.size(), fmt.raw_number),
             'size of the region'),
         Formatter(
             '<access rate>',
-            lambda index, region, fmt:
+            lambda index, region, snapshot, record, fmt:
             _damo_fmt_str.format_percent(region.nr_accesses.percent,
                                          fmt.raw_number),
             'monitored access rate of the region'),
+        Formatter(
+            '<access hz>',
+            lambda index, region, snapshot, record, fmt:
+            _damo_fmt_str.format_hz(
+                region.nr_accesses.in_hz(
+                    snapshot_monitoring_intervals(
+                        snapshot, record.intervals)[1]), fmt.raw_number),
+            'monitored access frequency of the region in hz'),
         Formatter(
             '<age>',
-            lambda index, region, fmt:
+            lambda index, region, snapshot, record, fmt:
             _damo_fmt_str.format_time_us(region.age.usec, fmt.raw_number),
             'how long the access pattern of the region has maintained'),
         Formatter(
             '<temperature>',
-            lambda index, region, fmt:
+            lambda index, region, snapshot, record, fmt:
             temperature_str(region, fmt.raw_number, fmt),
             'access temperature of the region'),
         Formatter(
             '<filters passed bytes>',
-            lambda index, region, fmt:
+            lambda index, region, snapshot, record, fmt:
             _damo_fmt_str.format_sz(region.sz_filter_passed, fmt.raw_number)
             if region.sz_filter_passed is not None else 'N/A',
             'bytes of the region that passed DAMOS filters'),
         Formatter(
             '<box>',
-            lambda index, region, fmt:
+            lambda index, region, snapshot, record, fmt:
             fmt.region_box_format.to_str(region),
             'user-customizable (via --region_box_*) box ' \
                     '(age/access_rate/size by default)'),
@@ -301,17 +314,46 @@ def damos_stats_str(snapshot, record, fmt):
         return 'none'
     return snapshot.damos_stats.to_str(fmt.raw_number)
 
-def infer_aggr_time_us(snapshot, record):
+def snapshot_monitoring_intervals(snapshot, record_intervals):
+    '''
+    Returns sampling and aggregation intervals for the snapshot, that may
+    auto-tuned.
+    '''
+    if not record_intervals.intervals_goal.enabled():
+        return [record_intervals.sample, record_intervals.aggr]
+
+    aggr_sample_ratio = record_intervals.aggr / record_intervals.sample
     if snapshot.sample_interval_us is not None:
-        aggr_sample_ratio = record.intervals.aggr / record.intervals.sample
-        return snapshot.sample_interval_us * aggr_sample_ratio
+        return [snapshot.sample_interval_us,
+                snapshot.sample_interval_us * aggr_sample_ratio]
 
     snapshot_aggr_us = (snapshot.end_time - snapshot.start_time) / 1000
-    record_aggr_us = record.intervals.aggr
+    record_aggr_us = record_intervals.aggr
     # if error is small enough, use simpler number
     if abs(snapshot_aggr_us - record_aggr_us) / record_aggr_us < 0.1:
-        return record_aggr_us
-    return snapshot_aggr_us
+        return [record_intervals.sample, record_aggr_us]
+    snapshot_sample_us = snapshot_aggr_us / aggr_sample_ratio
+    return [snapshot_sample_us, snapshot_aggr_us]
+
+def infer_aggr_time_us(snapshot, record):
+    return snapshot_monitoring_intervals(snapshot, record.intervals)[1]
+
+def max_access_hz(intervals):
+    max_nr_accesses = intervals.aggr / intervals.sample
+    aggr_sec = intervals.aggr / 1000000
+    return max_nr_accesses / aggr_sec
+
+def snapshot_intervals_str(snapshot, record, fmt):
+    intervals = record.intervals
+    aggr_sample_ratio = intervals.aggr / intervals.sample
+    snapshot_aggr_us = infer_aggr_time_us(snapshot, record)
+    snapshot_sample_us = snapshot_aggr_us / aggr_sample_ratio
+    max_access_hz = aggr_sample_ratio / (snapshot_aggr_us / 1000000)
+
+    return 'sample %s aggr %s (max access hz %s)' % (
+        _damo_fmt_str.format_time_us(snapshot_sample_us, fmt.raw_number),
+        _damo_fmt_str.format_time_us(snapshot_aggr_us, fmt.raw_number),
+        _damo_fmt_str.format_hz(max_access_hz, fmt.raw_number, no_suffix=True))
 
 def estimated_mem_bw(snapshot, record, fmt, filter_passed_only=False):
     access_bytes = 0
@@ -987,7 +1029,7 @@ def format_output(template, formatters, fmt, record, snapshot=None,
         elif formatters == snapshot_formatters:
             txt = formatter.format_fn(snapshot, record, fmt)
         elif formatters == region_formatters:
-            txt = formatter.format_fn(index, region, fmt)
+            txt = formatter.format_fn(index, region, snapshot, record, fmt)
         txt = apply_min_chars(fmt.min_chars_for, formatter.keyword, txt)
         template = template.replace(formatter.keyword, txt)
     template = template.replace('\\n', '\n')
@@ -1384,7 +1426,7 @@ def set_formats_handle_styles(fmt, args, records):
         fmt.format_snapshot_head = \
                 'snapshot time: [<start time>, <end time>] (<duration>)'
         fmt.format_region = \
-                '<box> size <size> access rate <access rate> age <age>'
+                '<box> size <size> access <access hz> age <age>'
         fmt.region_box_min_max_height = [1, 1]
         fmt.region_box_min_max_length = [1, 40]
         fmt.region_box_align = 'right'
@@ -1396,14 +1438,14 @@ def set_formats_handle_styles(fmt, args, records):
     elif args.style == 'temperature-percentiles':
         set_formats_percentiles(args, fmt, records, 'temperature')
     elif args.style == 'cold':
-        fmt.format_region = '<box> <size> access <access rate> <age>'
+        fmt.format_region = '<box> <size> access <access hz> <age>'
         fmt.region_box_min_max_height = [1, 1]
         fmt.region_box_min_max_length = [1, 40]
         fmt.region_box_align = 'right'
         fmt.region_box_colorset = 'emotion'
         fmt.sort_regions_by = ['temperature']
     elif args.style == 'hot':
-        fmt.format_region = '<box> <size> access <access rate> <age>'
+        fmt.format_region = '<box> <size> access <access hz> <age>'
         fmt.region_box_min_max_height = [1, 1]
         fmt.region_box_min_max_length = [1, 40]
         fmt.region_box_align = 'right'
@@ -1429,7 +1471,7 @@ def set_formats_record_default(fmt, records):
         else:
             fmt.format_record_head = ''
     if fmt.format_record_tail is None:
-        tail_lines = ['monitoring intervals: <intervals>']
+        tail_lines = ['record DAMON intervals: <intervals>']
         if intervals_goal_enabled(records):
             tail_lines.append('# <intervals goal>')
         fmt.format_record_tail = '\n'.join(tail_lines)
@@ -1448,6 +1490,7 @@ def set_formats_snapshot_default(fmt, records, args, ops_filters_installed):
                     '(<duration>)\n<heatmap>'
         else:
             fmt.format_snapshot_head = 'heatmap: <heatmap>'
+        fmt.format_snapshot_head += '\nintervals: <snapshot intervals>'
     if ops_filters_installed:
         fmt.format_snapshot_head += \
                 '\n# damos filters (df): <filters passed type>'
@@ -1474,7 +1517,7 @@ def set_formats_snapshot_default(fmt, records, args, ops_filters_installed):
 
 def set_formats_region_default(fmt, records, args):
     default_region_format = \
-            '<index> addr <start address> size <size> access <access rate> ' \
+            '<index> addr <start address> size <size> access <access hz> ' \
             'age <age>'
     if args.region_box:
         if fmt.region_box_min_max_height[1] > 1:
@@ -1759,7 +1802,7 @@ def add_fmt_args(parser, hide_help=False):
             '--temperature_weights', type=int, metavar='<int>', nargs=3,
             default=[0, 100, 100],
             help=' '.join(
-                ['temperature weights for size, access rate, and age',
+                ['temperature weights for size, access frequency, and age',
                  'of each region']) if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--hist_logscale', action='store_true',
                         help='draw histograms in logscale'
@@ -1861,6 +1904,7 @@ def add_fmt_args(parser, hide_help=False):
             default=[['<index>', 3],
                      ['<start address>', 12],['<end address>', 11],
                      ['<size>', 11], ['<access rate>', 5],
+                     ['<access hz>', 6],
                      ['<age>', 13],
                      ['<filters passed type>', 10],
                      ],
