@@ -6,6 +6,8 @@ cd "$bindir"
 
 damo="../../damo"
 
+cmd_log=$(mktemp damo_test_cmd_log_XXX)
+
 cleanup_files()
 {
 	files_to_remove="./damon.data ./damon.data.perf.data ./damon.data.old"
@@ -13,7 +15,7 @@ cleanup_files()
 	do
 		if [ -f "$file" ]
 		then
-			if ! rm "$file"
+			if ! sudo rm "$file"
 			then
 				echo "removing $file failed"
 				exit 1
@@ -25,10 +27,11 @@ cleanup_files()
 test_record_permission()
 {
 	sudo "$damo" record "sleep 3" --timeout 3 --output_permission 611 \
-		&> /dev/null
+		&> "$cmd_log"
 	if [ ! "$(stat -c %a damon.data)" = "611" ]
 	then
 		echo "FAIL record-permission"
+		cat "$cmd_log"
 		exit 1
 	fi
 	cleanup_files
@@ -66,34 +69,37 @@ test_record_validate()
 	then
 		sudo "$damo" record "$target" --timeout "$timeout" \
 			--damon_interface_DEPRECATED "$damon_interface" \
-			&> /dev/null
+			&> "$cmd_log"
 	else
 		sudo "$damo" record "$target" --timeout "$timeout" \
 			--regions "$regions_boundary" \
 			--damon_interface_DEPRECATED "$damon_interface" \
-			&> /dev/null
+			&> "$cmd_log"
 	fi
 
 	rc=$?
-	if [ $rc -ne 0 ] && [ $rc -ne 124 ]
+	if [ $rc -ne 0 ]
 	then
 		echo "FAIL $testname"
 		echo "(damo-record command failed with value $rc)"
+		cat "$cmd_log"
 		exit 1
 	fi
 
 	if [ "$regions_boundary" = "none" ]
 	then
-		if ! "$damo" validate &> /dev/null
+		if ! sudo "$damo" validate &> "$cmd_log"
 		then
 			echo "FAIL $testname (record file is not valid)"
+			cat "$cmd_log"
 			exit 1
 		fi
 	else
-		if ! "$damo" validate --regions_boundary "$regions_boundary" \
-			&> /dev/null
+		if ! sudo "$damo" validate \
+			--regions_boundary "$regions_boundary" &> "$cmd_log"
 		then
 			echo "FAIL $testname (record file is not valid)"
+			cat "$cmd_log"
 			exit 1
 		fi
 	fi
@@ -101,6 +107,7 @@ test_record_validate()
 	if [ -f ./damon.data.perf.data ]
 	then
 		echo "FAIL $testname (perf.data is not removed)"
+		cat "$cmd_log"
 		exit 1
 	fi
 
@@ -108,6 +115,7 @@ test_record_validate()
 	if [ ! "$permission" = "600" ]
 	then
 		echo "FAIL $testname (out file permission $permission)"
+		cat "$cmd_log"
 		exit 1
 	fi
 
@@ -149,4 +157,5 @@ fi
 
 test_record_permission
 
+rm -f "$cmd_log"
 echo "PASS $(basename $(pwd))"
