@@ -36,8 +36,8 @@ class TestDamon(unittest.TestCase):
         damos_kvpairs = damos.to_kvpairs()
         self.assertEqual(type(damos_kvpairs), collections.OrderedDict)
         self.assertEqual(list(damos_kvpairs.keys()),
-                ['action', 'access_pattern', 'apply_interval_us', 'quotas',
-                    'watermarks', 'filters', 'stats'])
+                ['action', 'dests', 'access_pattern', 'apply_interval_us',
+                 'quotas', 'watermarks', 'filters', 'stats'])
         self.assertEqual(list(damos_kvpairs['stats'].keys()),
                 ['nr_tried', 'sz_tried', 'nr_applied', 'sz_applied',
                  'sz_ops_filter_passed', 'qt_exceeds'])
@@ -50,15 +50,14 @@ class TestDamon(unittest.TestCase):
         ctx_kvpairs = ctx.to_kvpairs()
         self.assertEqual(type(ctx_kvpairs), collections.OrderedDict)
         self.assertEqual(list(ctx_kvpairs.keys()),
-                ['ops', 'addr_unit', 'targets', 'intervals', 'nr_regions',
-                 'schemes'])
+                ['ops', 'targets', 'intervals', 'nr_regions', 'schemes'])
         self.assertEqual(ctx, _damon.DamonCtx.from_kvpairs(ctx_kvpairs))
 
         kdamond = _damon.Kdamond('off', 123, [ctx])
         kdamond_kvpairs = kdamond.to_kvpairs()
         self.assertEqual(type(kdamond_kvpairs), collections.OrderedDict)
         self.assertEqual(list(kdamond_kvpairs.keys()),
-                ['state', 'pid', 'contexts'])
+                ['state', 'pid', 'refresh_ms', 'contexts'])
         self.assertEqual(kdamond,
             _damon.Kdamond.from_kvpairs(kdamond_kvpairs))
 
